/*===============================================================
 * Project: 
 * Author : Terry 'Mongoose' Hendrix
 * Website: http://www.westga.edu/~stu7440/
 * Email  : stu7440@westga.edu
 * Object : 
 * Comment: This strips documents out of [partial] disk images
 *          in a clumsy, yet effective way
 *
 *-- History -----------------------------------------------
 *
 * 2003.02.25:
 * Mongoose - Automated recovery assistance using system and dd
 *            ( I know, but it works )
 *
 * 2002.09.06:
 * Mongoose - RTF support
 *
 * 2002.08.31:
 * Mongoose - Created
 *
 ==============================================================*/

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <ctype.h>


#define DOC_HEADER 0xe011cfd0
#define RTF_HEADER 0x74725c7b

int findDocument(char *filename, char *base);

int getCount(char *filename)
{
	int count = -1, i = 0;
	char c;
	char buffer[64];

	FILE *f = fopen(filename, "rb");

	if (!f)
	{
		perror("Error: ");
		return -1;
	}

	memset(buffer, 0, 64);

	while (fscanf(f, "%c", &c) != EOF)
	{
		if (!isdigit(c))
		{
			i = 0;

			if (c == '+' && buffer[0])
			{
				count = atoi(buffer);
				// printf("%s => %i\n", buffer, count);
				return count;
			}
		}
		else
		{
			buffer[i++] = c;// printf("%c\n", c);
		}
	}

	fclose(f);

	return -1;
}


int autoRecover(char *imgName, char *base)
{
	unsigned int count = 1474560, read = 0, skip = 0, try = 0, total = 0;
	char *device = "/dev/fd0";
	char *pipe = "/tmp/rword.pipe";
	char image[64];
	char basename[64];
	char cmd[256];
	
	for (skip = 0; skip < count; ++try)
	{
		snprintf(image, 64, "%s-%i.out", imgName, try);
		snprintf(basename, 64, "%s-%i", base, try);

		snprintf(cmd, 256, "dd skip=%u count=%u obs=1 ibs=1 if=%s of=%s 2> %s", 
			skip, count - skip, device, image, pipe);

		system(cmd);

		read = getCount(pipe);
		skip = read + 4096;
		total += read;

		if (read == -1)
		{
			printf("Error: Couldn't get valid byte count, exiting\n");
			skip = count;
			break;
		}

		printf("Pass #%i: Read %i bytes, %i bytes total\n",
			try, read, total);

		if (read > 0)
		{
			printf("Recovering documents from %s...\n", image);
			findDocument(image, basename);
		}

		if (total > count)
		{
			printf("Error: Over read...\n");
			break;
		}
	}

	printf("Disk read in %i sections, %i bytes recovered\n", try, total);

	return 0;
}


int findDocument(char *filename, char *base)
{
	unsigned int token;
	FILE *f, *f2;
	int recover = 0;
	int i = 0;
	char outfile[128];
	char basename[64];


	if (!filename || !filename[0] || !base || !base[0])
	{
		return -1;
	}

	strncpy(basename, base, 63);

	f2 = NULL;

	f = fopen(filename, "rb");

	if (!f)
	{
		perror("File in open error: ");
		return -1;
	}

	while (fread(&token, 4, 1, f) == 1)
	{
		++i;

		if (token == DOC_HEADER || token == RTF_HEADER)
		{
			++recover;

			switch (token)
			{
			case DOC_HEADER:
				printf("Word DOC header found!\n");
				snprintf(outfile, 127, "%s-%05i.doc", 
					basename, recover);
				break;
			case RTF_HEADER:
				printf("RTF header found!\n");
				snprintf(outfile, 127, "%s-%05i.rtf", 
					basename, recover);
				break;
			}

			printf("\tRecovering as '%s'\n\n", outfile);

			if (f2)
			{
				fclose(f2);
			}

			f2 = fopen(outfile, "wb");

			if (!f2)
			{
				perror("File out open error: ");
				fclose(f);
				return -2;
			}
		}


		switch (recover)
		{
		case 0:
			break;
		default:
			fwrite(&token, 4, 1, f2);
			break;
		}
	}

	printf("%i RTF/DOC headers found in %i bytes\n", recover, i);

	if (f2)
	{
		fclose(f2);
	}

	fclose(f);

	return 0;
}


int main(int argc, char *argv[])
{
	int args = 0;

	if (argc > 1) args = 1;

	if (argc > 2) args = 2;

	if (argc > 3) args = 3;

	switch (args)
	{
	case 3:
		if (strncmp(argv[1], "-auto", 5) == 0 ||
			strncmp(argv[1], "-a", 2) == 0)
		{
			return autoRecover(argv[3], argv[2]);
		}
		else
		{
			printf("No -auto request\n");
		}
		break;
	case 2:
		return findDocument(argv[2], argv[1]);
		break;
	default:
		printf("\n%s [-auto] FILENAME_BASE IMAGE_NAME.IMG\n", 
			argv[0]);
	}

	return 0;
}

